/*
 *        spAudioP.h
 */

#ifndef __SPAUDIOP_H
#define __SPAUDIOP_H

#include <sp/spAudio.h>
#include <sp/spPlugin.h>

#if defined(ANDROID)
#undef OSS
#elif defined(linux) && !defined(OSS)
#define OSS
#endif

#if defined(SGI)
#include <dmedia/audio.h>
#elif defined(MACOSX)
#if defined(MACOSX64)
#include <CoreAudio/CoreAudioTypes.h>
#if !defined(IPHONE)
#include <CoreAudio/AudioHardware.h>
#endif
#endif
#if !defined(MACOSX64) && !defined(IPHONE)
#include <Carbon/Carbon.h>
#endif
#elif defined(MACOS)
#include <Sound.h>
#elif defined(_WIN32)
#include <windows.h>
#elif defined(ANDROID)
#include <SLES/OpenSLES.h>
#include <SLES/OpenSLES_Android.h>
#elif defined(sun)
#include <sys/stropts.h>
#include <sys/filio.h>
#ifndef SOLARIS
#include <multimedia/libaudio.h>
#include <multimedia/audio_device.h>
#else
#include <sys/audioio.h>
#endif
#endif

#if defined(MACOS)

#define SP_MAC_SUPPORT_AUDIO_INPUT

/*#if defined(MACOSX) || defined(IPHONE)*/
#if defined(MACOSX64) || defined(IPHONE)
#define SP_MAC_USE_AUDIO_UNIT
#endif

#if defined(SP_MAC_USE_DOUBLE_BUFFER) && TARGET_API_MAC_CARBON
#include "CarbonSndPlayDB.h"
#endif

#if defined(SP_MAC_USE_AUDIO_UNIT)
#include <AudioUnit/AudioUnit.h>
#include <AudioToolbox/AudioConverter.h>
#endif

#endif

#ifdef __cplusplus
extern "C" {
#endif

#define SP_AUDIO_DEFAULT_INTERNAL_BUFFER_SIZE 8192
    
#define SP_DEFAULT_AUDIO_BUFFER_SIZE 8192
#if defined(MACOSX) || defined(ANDROID)
#define SP_DEFAULT_NUM_BUFFER 8
#elif defined(MACOS)
#define SP_DEFAULT_NUM_BUFFER 64
#else
#define SP_DEFAULT_NUM_BUFFER 64
#endif

#if defined(_WIN32)    
typedef struct _spWinAudio *spWinAudio;
#endif

typedef struct _spAudioFuncList
{
    spBool (*init_driver) __P((spAudio audio, const char *driver_name, void *data));
    spBool (*free_driver) __P((spAudio audio));
    void (*terminate) __P((void));

    spBool (*get_num_device) __P((spAudio audio, int *num_device));
    const char *(*get_device_name) __P((spAudio audio, int index));
    spBool (*select_device) __P((spAudio audio, int index));

    unsigned long (*get_caps) __P((spAudio audio));
    spBool (*display_option_dialog) __P((spAudio audio));
    
    spBool (*set_buffer_size) __P((spAudio audio));
    spBool (*set_num_buffer) __P((spAudio audio));
    spBool (*set_blocking_mode) __P((spAudio audio));
    spAudioCallbackType (*set_callback_func) __P((spAudio audio));
    
    spBool (*set_sample_bit) __P((spAudio audio));
    spBool (*set_sample_rate) __P((spAudio audio));
    spBool (*set_channel) __P((spAudio audio));
    
    spBool (*open_device) __P((spAudio audio, const char *mode));
    spBool (*close_device) __P((spAudio audio));
    spBool (*get_output_position) __P((spAudio audio, spLong *position));
    spBool (*stop) __P((spAudio audio));
    
    long (*write) __P((spAudio audio, char *data, long length));
    spBool (*sync) __P((spAudio audio));

    long (*read) __P((spAudio audio, char *data, long length));
} spAudioFuncList;

struct _spAudio {
    int samp_bit;
    double samp_rate;
    int num_channel;
    int buffer_size;
    long samp_byte;
    int duplex_flag;
    int only_flag;
    int block_mode;
    int specified_samp_bit;
    spULong input_total;
    spULong output_total;
    int num_buffer;
#if defined(SP_SUPPORT_AUDIO)
#if defined(OSS)
    long device_index;
    int fragment_size;
    int prev_fd;
    int input_fd;
    int output_fd;
    int fd0;
    int samp_bit0;
    int samp_rate0;
    int num_channel0;
    int block_size0;
    int fd1;
    int samp_bit1;
    int samp_rate1;
    int num_channel1;
    int block_size1;
#elif defined(SGI)
    ALconfig audio_config;
    ALport input_port;
    ALport output_port;
#elif defined(_WIN32)
    UINT input_device_id;
    UINT output_device_id;
    spWinAudio input_audio;
    spWinAudio output_audio;
#elif defined(ANDROID)
    SLObjectItf engineObject;
    SLEngineItf engineEngine;
    SLObjectItf outputMixObject;
    
    volatile spBool doStop;
    void *playerMutex;
    void *playerEvent;
    SLObjectItf bqPlayerObject;
    SLPlayItf bqPlayerPlay;
    SLAndroidSimpleBufferQueueItf bqPlayerBufferQueue;
    volatile spBool playerStarted;
    char *playerBuffer;
    long playerBufferSize;
    long playerNumBlock;
    long playerBlockSize;
    volatile long playerFillCount;
    volatile long playerBufferOffset;
    volatile long playerCopyOffset;
    volatile spLong playerTotalWrite;

    void *recorderMutex;
    void *recorderEvent;
    SLObjectItf recorderObject;
    SLRecordItf recorderRecord;
    SLAndroidSimpleBufferQueueItf recorderBufferQueue;
    volatile spBool recorderStarted;
    char *recorderBuffer;
    long recorderBufferSize;
    long recorderNumBlock;
    long recorderBlockSize;
    volatile long recorderFillCount;
    volatile long recorderBufferOffset;
    volatile long recorderCopyOffset;
#elif defined(sun)
    int input_fd;
    int output_fd;
    audio_info_t audio_info;
#elif defined(SP_MAC_USE_AUDIO_UNIT)
    volatile spBool doStop;
    spBool inputUnitInitialized;
    spBool outputUnitInitialized;
    spBool inputUnitStarted;
    spBool outputUnitStarted;
    spBool inputEnabled;
    spBool outputEnabled;

    void *inputEvent;
    void *outputEvent;
    void *inputMutex;
    void *outputMutex;
    
    AudioUnit inputUnit;
    AudioUnit outputUnit;
#if defined(IPHONE)
    AudioComponentDescription cd;
    AudioComponent component;
#else
    AudioObjectID inputDevice;
    AudioObjectID outputDevice;
#if MAC_OS_X_VERSION_MAX_ALLOWED >= 1060
    AudioComponentDescription cd;
    AudioComponent component;
#else
    ComponentDescription cd;
    Component component;
#endif
#endif
    AudioStreamBasicDescription systemFormat;
    AudioStreamBasicDescription currentFormat;
    AudioConverterRef inputConverter;
    AudioConverterRef outputConverter;

    Float64 inputSampleTime;
    Float64 outputSampleTime;
    Float64 inputInitialSampleTime;
    Float64 outputInitialSampleTime;
    spLong prevCallbackedOutputPosition;
    
    AudioBufferList *inputDeviceBufferList;
    AudioBufferList *inputConversionBufferList;
    AudioBufferList *outputDeviceBufferList;
    
    char *inputBuffer;
    UInt32 inputDeviceBufferAllocSize;
    UInt32 inputDeviceBufferSize;
    UInt32 inputBufferSize;
    UInt32 bufferSizeForInput;
    
    volatile UInt32 inputBufferOffset;
    volatile UInt32 inputBufferFilled;
    UInt32 inputBufferReadPos;
    UInt32 inputCallbackCount;
    UInt32 inputConvertedFrames;
    UInt32 inputConvertedOffset;
    spBool inputWaiting;
    
    char *inputConversionBuffer;
    
    char *outputBuffer;
    UInt32 outputDeviceBufferAllocSize;
    UInt32 outputDeviceBufferSize;
    UInt32 outputBufferSize;
    UInt32 bufferSizeForOutput;
    
    volatile UInt32 outputBufferOffset;
    volatile UInt32 outputBufferFilled;
    UInt32 outputBufferWritePos;
    UInt32 outputCallbackCount;
    UInt32 outputConvertedFrames;
    UInt32 outputConvertedOffset;
    spBool outputWaiting;
    
    char *outputConversionBuffer;
    
#elif defined(MACOS)
    int support_input;
    int support_output;
    int support_16bit;
    int stereo_input;
    int stereo_output;

    SndChannelPtr sndChannel;
    ExtSoundHeaderPtr sndHeader;
#if defined(SP_MAC_USE_DOUBLE_BUFFER)
    SndDoubleBufferHeaderPtr sndDoubleHeader;
    SndDoubleBackUPP sndDoubleBackUPP;
#else
    SndCallBackUPP sndCallBackUPP;
#endif

    spLong prevCallbackedOutputPosition;
    
    long playDataSize;
    Ptr playDataBuffer;
    
    int playState;
    long playOffset;
    long playDataOffset;
    long playDataFilled;
    
#if defined(SP_MAC_SUPPORT_AUDIO_INPUT)
    SPBPtr pspb;
    SICompletionUPP siCompletionUPP;
    long refNum;
    long recBufferSize;
    
    long recDataSize;
    Ptr recDataBuffer;

    spBool recStarted;
    long recOffset;
    long recDataOffset;
    long recDataFilled;

    SoundConverter recsc;
    unsigned long recConvInputFrames;
    unsigned long recConvBufferSize;
    Ptr recConvBuffer;
    Ptr recConvOutputBuffer;
#endif
#endif
#endif

    char *internal_buffer;
    long internal_buffer_size;
    
    spAudioCallbackType call_type;
    spAudioCallbackType supported_call_type;
    spAudioCallbackFunc call_func;
    void *call_data;

    char default_device_name[SP_MAX_LINE];
    
    spAudioFuncList func_list;
    
    spPlugin *plugin;
};

#if defined(MACOS)
#pragma import on
#endif

extern int spGetNumAudioDriverArch(void);
extern const char *spGetAudioDriverNameArch(int index);
extern spBool spInitAudioDriverArch(spAudio audio, const char *driver_name, void *data);
extern spBool spGetNumAudioDeviceArch(spAudio audio, int *num_device);
extern const char *spGetAudioDeviceNameArch(spAudio audio, int device_index);
extern spBool spSelectAudioDeviceArch(spAudio audio, int device_index);
extern spBool spSetAudioSampleBitArch(spAudio audio);
extern spBool spSetAudioSampleRateArch(spAudio audio);
extern spBool spSetAudioChannelArch(spAudio audio);
extern spBool spSetAudioBufferSizeArch(spAudio audio);
extern spBool spSetAudioNumBufferArch(spAudio audio);
extern spBool spSetAudioBlockingModeArch(spAudio audio);
extern spAudioCallbackType spSetAudioCallbackFuncArch(spAudio audio);
extern spBool spOpenAudioDeviceArch(spAudio audio, const char *mode);
extern spBool spOpenInputAudioDeviceArch(spAudio audio);
extern spBool spOpenOutputAudioDeviceArch(spAudio audio);
extern spBool spCloseAudioDeviceArch(spAudio audio);
extern long spReadAudioArch(spAudio audio, char *data, long length);
extern long spWriteAudioArch(spAudio audio, char *data, long length);
extern spBool spGetAudioOutputPositionArch(spAudio audio, spLong *position);
extern spBool spStopAudioArch(spAudio audio);
extern spBool spSyncAudioArch(spAudio audio);
extern spBool spFreeAudioDriverArch(spAudio audio);
extern void spTerminateAudioArch(void);

#if defined(MACOS)
#pragma import off
#endif

#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#endif /* __SPAUDIOP_H */
