/** @file sp/spOutputPlugin.h
 */

#ifndef __SPOUTPUTPLUGIN_H
#define __SPOUTPUTPLUGIN_H

#include <sp/spWave.h>
#include <sp/spPlugin.h>

#ifdef __cplusplus
extern "C" {
#endif

/** @~english @defgroup spOutputPluginGroup Output Plugin */

/** @addtogroup spOutputPluginGroup
 *  @{ */

/** @~english
 *  @class spOutputPlugin spOutputPlugin.h <sp/spOutputPlugin.h>
 *  @extends spPlugin
 *  @brief A class to handle plugin-based output to an audio file.
@details Actually, spOutputPlugin is not a class. You can handle this by using spPlugin type typedefed by `typedef struct _spPlugin spPlugin;`.
 *  @sa @ref spPluginForAudioGroup , spInputPlugin
 */

/*
 *  - Internal bits/sample must be greater than 16.
 *  - If bits/sample = 8 or 16, internal data must be short format.
 *  - If bits/sample = 33, it means float data.
 *  - If bits/sample >= 33, internal data must be double format (case without SP_PLUGIN_CAPS_NO_FLOAT_CONVERSION).
 *  - If bits/sample = 24 or 32, internal data must be long format.
 */
    
#define SP_PLUGIN_OUTPUT "output 1.3"

/* Version 1.6.1+ */
#define SP_PLUGIN_CAPS_REPLACE_SONG_INFO (1<<12) /* can replace song info by set_song_info */
/* Version 1.7.1+ */
#define SP_PLUGIN_CAPS_NO_FLOAT_CONVERSION (1<<13)
/* Version 1.7.2+ */
#define SP_PLUGIN_CAPS_LIVE_WRITE (1<<14)
    
/* plugin information ID */
#define SP_PLUGIN_INFO_SONG_INFO_MASK "song_info_mask"
#define SP_PLUGIN_INFO_BIT_RATE "bit_rate"
#define SP_PLUGIN_INFO_HEADER_LENGTH "header_length"
#define SP_PLUGIN_INFO_MPEG_DECODE_OPTION "mpeg_decode_option"
#define SP_PLUGIN_INFO_NUM_ORDER "num_order"
#define SP_PLUGIN_INFO_MAX_CHANNEL "max_channel"

#define SP_PLUGIN_INFO_NUM_BUFFER "num_buffer"
#define SP_PLUGIN_INFO_BLOCKING_MODE "blocking_mode"

#define SP_PLUGIN_INFO_DIALOG_RECT "dialog_rect"
#define SP_PLUGIN_INFO_DIALOG_CREATE "dialog_create"
#define SP_PLUGIN_INFO_DIALOG_END "dialog_end"
#define SP_PLUGIN_INFO_DIALOG_DESTROY "dialog_destroy"
    
/* for backwards compatibility */
#define SP_BIT_RATE_ID SP_PLUGIN_INFO_BIT_RATE
#define SP_HEADER_LENGTH_ID SP_PLUGIN_INFO_HEADER_LENGTH
#define SP_MPEG_DECODE_OPTION_ID SP_PLUGIN_INFO_MPEG_DECODE_OPTION
    
#define SP_PLUGIN_INFO_CALLBACK_FUNC "device_callback_func"
#define SP_PLUGIN_INFO_CALLBACK_TYPE "device_callback_type"
#define SP_PLUGIN_INFO_CALLBACK_DATA "device_callback_data"
#define SP_PLUGIN_INFO_CALLBACK_HOST_DATA "device_callback_host_data"

    
#define SP_PLUGIN_DEVICE_NO_CALLBACK 0
#define SP_PLUGIN_DEVICE_OUTPUT_POSITION_CALLBACK (1<<0) /* data1: long *position, data2: NULL */
#define SP_PLUGIN_DEVICE_OUTPUT_BUFFER_CALLBACK (2<<0) /* data1: void *buffer, data2: long *buffer_length */

    
typedef struct _spOutputPluginRec spOutputPluginRec;

/** @} */

#if defined(MACOS)
#pragma import on
#endif

extern spBool spIsIoPlugin(spPlugin *plugin);
extern spBool spIsOutputPlugin(spPlugin *plugin);
extern spPluginDeviceType spGetPluginDeviceType(spPlugin *plugin);

/* file type must be set before other file-rerated settings such as sample rate, bits/sample, etc. */
extern int spFindPluginFileTypeIndex(spPlugin *plugin, const char *type);
extern spBool spSetPluginFileTypeIndex(spPlugin *plugin, int index);
extern int spGetPluginFileTypeIndex(spPlugin *plugin);
extern const char *spGetPluginFileTypeString(spPlugin *plugin, int index);
extern const char *spGetPluginFileDescString(spPlugin *plugin, int index);
extern const char *spGetPluginFileFilterString(spPlugin *plugin, int index);
extern spBool spSetPluginFileType(spPlugin *plugin, const char *type);
extern char *xspGetPluginFileType(spPlugin *plugin, spBool long_flag);
extern char *xspGetPluginFileDescription(spPlugin *plugin, spBool filter_flag);
extern char *xspGetPluginFileFilter(spPlugin *plugin);
extern spBool spSearchPluginFileType(int index, spPluginType plugin_type,
                                     spPluginDeviceType device_type, char *plugin_name,
                                     char *file_type, char *file_desc, char *file_filter);
extern spBool spSearchPluginDevice(int index, spPluginType plugin_type,
                                   spPluginDeviceType device_type, char *plugin_name);
extern spBool spSetPluginSongInfo(spPlugin *plugin, spSongInfo *song_info);
extern spBool spGetPluginSongInfo(spPlugin *plugin, spSongInfo *song_info);
extern spBool spGetPluginSongInfoMask(spPlugin *plugin, unsigned long *info_mask); /* mask doesn't include ID3v2's support masks if the mask includes flag of ID3v2 such as SP_SONG_ID3V2_4_MASK. */
extern const char *spGetPluginBestSuffix(spPlugin *plugin);
extern spBool spIsSupportedByPlugin(spPlugin *plugin, const char *filename);

extern spBool spGetPluginNumDevice(spPlugin *plugin, int *num_device);
extern const char *spGetPluginDeviceName(spPlugin *plugin, int index);
extern spBool spSelectPluginDevice(spPlugin *plugin, int index);
extern spBool spSetPluginBufferSize(spPlugin *plugin, int buffer_size);
extern spBool spGetPluginBufferSize(spPlugin *plugin, int *buffer_size);
extern spBool spSetPluginVolume(spPlugin *plugin, int channel, int volume);
extern spBool spGetPluginVolume(spPlugin *plugin, int channel, int *volume);
extern spBool spSetPluginVolumeStereo(spPlugin *plugin, int l_volume, int r_volume);
extern spBool spGetPluginVolumeStereo(spPlugin *plugin, int *l_volume, int *r_volume);

extern spBool spSetPluginSampleBit(spPlugin *plugin, int samp_bit);
extern spBool spGetPluginSampleBit(spPlugin *plugin, int *samp_bit);
extern spBool spGetPluginDataSampleByte(spPlugin *plugin, int *samp_byte);
extern spBool spGetPluginDataSampleBit(spPlugin *plugin, int *samp_bit);
extern spBool spSetPluginChannel(spPlugin *plugin, int num_channel);
extern spBool spGetPluginChannel(spPlugin *plugin, int *num_channel);
extern spBool spSetPluginSampleRate(spPlugin *plugin, double samp_rate);
extern spBool spGetPluginSampleRate(spPlugin *plugin, double *samp_rate);
extern spBool spSetPluginOtherInfo(spPlugin *plugin, const char *id, void *data);
extern spBool spGetPluginOtherInfo(spPlugin *plugin, const char *id, void *data);

extern spPluginError spOpenPlugin(spPlugin *plugin, const char *filename, const char *mode);
extern spBool spClosePlugin(spPlugin *plugin);
extern spPluginState spGetPluginState(spPlugin *plugin);
extern spLong spGetPluginCurrentPosition(spPlugin *plugin);
extern spBool spStopPlugin(spPlugin *plugin);
extern spBool spPausePlugin(spPlugin *plugin);
extern spBool spRestartPlugin(spPlugin *plugin);

/** @~english Write data to a file by using a plugin.
The difference from spWritePluginInByte() is how to specify the size of data.
 *  @param[in] plugin spPlugin pointer obtained by spOpenFilePlugin() .
 *  @param[in] data Data to write. The type of data depends on bits/sample of data; short for 8/16 bits, long for 24/32 bits, and double for 64 bits.
 *  @param[in] length The length of \p data . Note that this \p length is not identical to duration in points.
For example, \p length is twice of duration in points of \p data for stereo data.
 *  @return Written length. This value is the same as \p length if successful.
 *  @relatesalso spOutputPlugin */
extern long spWritePlugin(spPlugin *plugin, void *data, long length);
/** @~english Write data to a file by using a plugin.
The difference from spWritePlugin() is how to specify the size of data.
 *  @param[in] plugin spPlugin pointer obtained by spOpenFilePlugin() .
 *  @param[in] data Data to write. The type of data depends on bits/sample of data; short for 8/16 bits, long for 24/32 bits, and double for 64 bits.
 *  @param[in] data_size The size of \p data in bytes. 
 *  @return Written size in bytes. This value is the same as \p data_size if successful.
 *  @relatesalso spOutputPlugin */
extern long spWritePluginInByte(spPlugin *plugin, void *data, long data_size);
/** @~english Write data to a file by using a plugin. The value of the data must be basically from -1 to 1 inclusive.
 *  @param[in] plugin spPlugin pointer obtained by spOpenFilePlugin() .
 *  @param[in] data Data to write. 
 *  @param[in] length The length of \p data . Note that this \p length is not identical to duration in points.
For example, \p length is twice of duration in points of \p data for stereo data.
 *  @param[in] weight This value is multiplied to the data before writing.
 *  @return Written length. This value is the same as \p length if successful.
 *  @relatesalso spOutputPlugin */
extern long spWritePluginDoubleWeighted(spPlugin *plugin, double *data, long length, double weight);
/** @~english Write data to a file by using a plugin. The value of the data must be basically from -1 to 1 inclusive.
 *  @param[in] plugin spPlugin pointer obtained by spOpenFilePlugin() .
 *  @param[in] data Data to write. 
 *  @param[in] length The length of \p data . Note that this \p length is not identical to duration in points.
For example, \p length is twice of duration in points of \p data for stereo data.
 *  @return Written length. This value is the same as \p length if successful.
 *  @relatesalso spOutputPlugin */
extern long spWritePluginDouble(spPlugin *plugin, double *data, long length);
extern spBool spFlushPlugin(spPlugin *plugin);

/** @~english Get the name of the related plugin. This function can be used to search an output plugin associated with an input plugin.
 *  @param[in] plugin_name The name of the plugin such as `"input_wav"` .
 *  @return The name of the related plugin such as `"output_wav"`  for `"input_wav"` as \p plugin_name . You have to free the memory by xspFree() .
 *  @relatesalso spOutputPlugin */
extern char *xspFindRelatedPluginFile(const char *plugin_name);
/** @~english Get the name of a plugin which is suitable for opening the file with the file name and the mode.
 *  @param[in] device_type Normally, specify SP_PLUGIN_DEVICE_FILE.
 *  @param[in] filename The name of the file.
 *  @param[in] mode The mode in opening the file such as `"r"` for reading mode and `"w"` for writing mode.
 *  @param[in,out] current_priority The address of the variable to hold the priority value.
This function searches the suitable plugin from this priority toward lower priority; ignores plugins which have higher priority than this value.
 *  @return The name of the suitable plugin. If the suitable plugin cannot be found, NULL will be returned. You have to free the memory by xspFree() . 
 *  @relatesalso spOutputPlugin */
extern char *xspFindSuitablePluginFileWithPriority(spPluginDeviceType device_type,
                                                   const char *filename, const char *mode, int *current_priority);
/** @~english Get the name of a plugin which is suitable for opening the file with the file name and the mode.
 *  @param[in] device_type Normally, specify SP_PLUGIN_DEVICE_FILE.
 *  @param[in] filename The name of the file.
 *  @param[in] mode The mode in opening file such as `"r"` for reading mode and `"w"` for writing mode.
 *  @return The name of the suitable plugin. If the suitable plugin cannot be found, NULL will be returned. You have to free the memory by xspFree() . 
 *  @relatesalso spOutputPlugin */
extern char *xspFindSuitablePluginFile(spPluginDeviceType device_type, const char *filename, const char *mode);
/** @~english Open the file by using a plugin.
 *  @param[in] plugin_name The name of the plugin. If you specify NULL, this funcion automatically detects the plugin name which is suitable for opening the file
by the file information for reading mode or the file name for writing mode, 
 *  @param[in] filename The name of the file to open.
 *  @param[in] mode The mode in opening the file such as `"r"` for reading mode and `"w"` for writing mode.
 *  @param[in] device_type Specify SP_PLUGIN_DEVICE_FILE for file I/O, SP_PLUGIN_DEVICE_AUDIO for audio device I/O.
 *  @param[in,out] wave_info The address of the variable of spWaveInfo type.
For reading mode, the returned variable holds information of the file. For writing mode, the file information is updated by using the variable content.
 *  @param[in,out] song_info The address of the variable of spSongInfo type. For reading mode, the returned variable holds song information of the file.
 *  @param[in] call_func The address of callback function. This function must have the following signature: @code
    spBool func(spPlugin *plugin, spPluginCallbackReason reason, void *host_data, void *call_data);
 *  @endcode    
 *  @param[in] call_data The address of callback user data. This data can be received by the 4-th argument of the callback function.
 *  @param[out] error The error code if the returned value is NULL .
 *  @return The pointer to spPlugin type. This pointer will be used for function calls following after this function call.
 *  @relatesalso spOutputPlugin */
extern spPlugin *spOpenFilePlugin(const char *plugin_name, const char *filename, const char *mode,
                                  spPluginDeviceType device_type,
                                  spWaveInfo *wave_info, spSongInfo *song_info,
                                  spPluginOpenCallback call_func, void *call_data,
                                  spPluginError *error);
/** @~english Open the file by using a plugin. This function firstly ignores \p plugin_name argument, then opens the file with the plugin having \p plugin_name only if this function cannot find the suitable plugin. This behavior is useful for reading a raw binary file only if all plugins cannot read the file.
 *  @param[in] plugin_name The name of the plugin. If you specify NULL, this funcion automatically detects the plugin name which is suitable for opening the file
by the file information for reading mode or the file name for writing mode, 
 *  @param[in] filename The name of the file to open.
 *  @param[in] mode The mode in opening the file such as `"r"` for reading mode and `"w"` for writing mode.
 *  @param[in] device_type Specify SP_PLUGIN_DEVICE_FILE for file I/O, SP_PLUGIN_DEVICE_AUDIO for audio device I/O.
 *  @param[in,out] wave_info The address of the variable of spWaveInfo type.
For reading mode, the returned variable holds information of the file. For writing mode, the file information is updated by using the variable content.
 *  @param[in,out] song_info The address of the variable of spSongInfo type. For reading mode, the returned variable holds song information of the file.
 *  @param[in] call_func The address of callback function. This function must have the following signature: @code
    spBool func(spPlugin *plugin, spPluginCallbackReason reason, void *host_data, void *call_data);
 *  @endcode    
 *  @param[in] call_data The address of callback user data. This data can be received by the 4-th argument of the callback function.
 *  @param[out] error The error code if the returned value is NULL .
 *  @return The pointer to spPlugin type. This pointer will be used for function calls following after this function call.
 *  @relatesalso spOutputPlugin */
extern spPlugin *spOpenFilePluginAuto(const char *plugin_name, const char *filename, const char *mode,
                                      spPluginDeviceType device_type,
                                      spWaveInfo *wave_info, spSongInfo *song_info,
                                      spPluginOpenCallback call_func, void *call_data,
                                      spPluginError *error);
/** @~english Open the file by using a plugin. This function firstly ignores \p plugin_name argument, then opens the file with the plugin having \p plugin_name only if this function cannot find the suitable plugin. This behavior is useful for reading a raw binary file only if all plugins cannot read the file.
 *  @param[in] plugin_name The name of the plugin. If you specify NULL, this funcion automatically detects the plugin name which is suitable for opening the file
by the file information for reading mode or the file name for writing mode, 
 *  @param[in] filename The name of the file to open.
 *  @param[in] mode The mode in opening the file such as `"r"` for reading mode and `"w"` for writing mode.
 *  @param[in] device_type Specify SP_PLUGIN_DEVICE_FILE for file I/O, SP_PLUGIN_DEVICE_AUDIO for audio device I/O.
 *  @param[in,out] wave_info The address of the variable of spWaveInfo type.
For reading mode, the returned variable holds information of the file. For writing mode, the file information is updated by using the variable content.
 *  @param[in,out] song_info The address of the variable of spSongInfo type. For reading mode, the returned variable holds song information of the file.
 *  @param[in] argc The number of elements of the array associated with \p argv .
 *  @param[in] argv An array that each element is a string specified as a command-line argument.
 *  @param[out] error The error code if the returned value is NULL .
 *  @return The pointer to spPlugin type. This pointer will be used for function calls following after this function call.
 *  @relatesalso spOutputPlugin */
extern spPlugin *spOpenFilePluginArg(const char *plugin_name, const char *filename, const char *mode,
                                     spPluginDeviceType device_type,
                                     spWaveInfo *wave_info, spSongInfo *song_info,
                                     int argc, char **argv, spPluginError *error);
/** @~english Open the file by using a plugin.
 *  @param[in] plugin_name The name of the plugin. If you specify NULL, this funcion automatically detects the plugin name which is suitable for opening the file
by the file information for reading mode or the file name for writing mode, 
 *  @param[in] filename The name of the file to open.
 *  @param[in] mode The mode in opening the file such as `"r"` for reading mode and `"w"` for writing mode.
 *  @param[in] device_type Specify SP_PLUGIN_DEVICE_FILE for file I/O, SP_PLUGIN_DEVICE_AUDIO for audio device I/O.
 *  @param[in,out] wave_info The address of the variable of spWaveInfo type.
For reading mode, the returned variable holds information of the file. For writing mode, the file information is updated by using the variable content.
 *  @param[in,out] song_info The address of the variable of spSongInfo type. For reading mode, the returned variable holds song information of the file.
 *  @param[in] argc The number of elements of the array associated with \p argv .
 *  @param[in] argv An array that each element is a string specified as a command-line argument.
 *  @param[out] error The error code if the returned value is NULL .
 *  @return The pointer to spPlugin type. This pointer will be used for function calls following after this function call.
 *  @relatesalso spOutputPlugin */
extern spPlugin *spOpenFilePluginArgAuto(const char *plugin_name, const char *filename, const char *mode,
                                         spPluginDeviceType device_type,
                                         spWaveInfo *wave_info, spSongInfo *song_info,
                                         int argc, char **argv, spPluginError *error);
/** @~english Close the file opened by using a plugin.
Since this function also frees memory of the pointer of spPlugin , don't access the pointer of spPlugin after calling this function.
 *  @param[in] plugin spPlugin pointer obtained by spOpenFilePlugin() .
 *  @retval SP_TRUE Success  @retval SP_FALSE Failure
 *  @relatesalso spOutputPlugin */
extern spBool spCloseFilePlugin(spPlugin *plugin);

#if defined(MACOS)
#pragma import off
#endif

#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#endif /* __SPOUTPUTPLUGIN_H */
